@@ -147,11 +147,12 @@ static ssize_t request_reader_entries(
                 char *buf,
                 size_t max) {
 
-        RequestMeta *m = cls;
+        RequestMeta *m = ASSERT_PTR(cls);
+        dual_timestamp previous_ts = DUAL_TIMESTAMP_NULL;
+        sd_id128_t previous_boot_id = SD_ID128_NULL;
         int r;
         size_t n, k;
 
-        assert(m);
         assert(buf);
         assert(max > 0);
         assert(pos >= m->delta);
@@ -223,7 +224,7 @@ static ssize_t request_reader_entries(
                 }
 
                 r = show_journal_entry(m->tmp, m->journal, m->mode, 0, OUTPUT_FULL_WIDTH,
-                                   NULL, NULL, NULL);
+                                   NULL, NULL, NULL, &previous_ts, &previous_boot_id);
                 if (r < 0) {
                         log_error_errno(r, "Failed to serialize item: %m");
                         return MHD_CONTENT_READER_END_WITH_ERROR;
@@ -255,7 +256,7 @@ static ssize_t request_reader_entries(
         errno = 0;
         k = fread(buf, 1, n, m->tmp);
         if (k != n) {
-                log_error("Failed to read from file: %s", errno != 0 ? strerror_safe(errno) : "Premature EOF");
+                log_error("Failed to read from file: %s", STRERROR_OR_EOF(errno));
                 return MHD_CONTENT_READER_END_WITH_ERROR;
         }
 
@@ -359,12 +360,10 @@ static mhd_result request_parse_arguments_iterator(
                 const char *key,
                 const char *value) {
 
-        RequestMeta *m = cls;
+        RequestMeta *m = ASSERT_PTR(cls);
         _cleanup_free_ char *p = NULL;
         int r;
 
-        assert(m);
-
         if (isempty(key)) {
                 m->argument_parse_error = -EINVAL;
                 return MHD_NO;
@@ -467,11 +466,10 @@ static int request_handler_entries(
                 void *connection_cls) {
 
         _cleanup_(MHD_destroy_responsep) struct MHD_Response *response = NULL;
-        RequestMeta *m = connection_cls;
+        RequestMeta *m = ASSERT_PTR(connection_cls);
         int r;
 
         assert(connection);
-        assert(m);
 
         r = open_journal(m);
         if (r < 0)
@@ -541,11 +539,10 @@ static ssize_t request_reader_fields(
                 char *buf,
                 size_t max) {
 
-        RequestMeta *m = cls;
+        RequestMeta *m = ASSERT_PTR(cls);
         int r;
         size_t n, k;
 
-        assert(m);
         assert(buf);
         assert(max > 0);
         assert(pos >= m->delta);
@@ -603,7 +600,7 @@ static ssize_t request_reader_fields(
         errno = 0;
         k = fread(buf, 1, n, m->tmp);
         if (k != n) {
-                log_error("Failed to read from file: %s", errno != 0 ? strerror_safe(errno) : "Premature EOF");
+                log_error("Failed to read from file: %s", STRERROR_OR_EOF(errno));
                 return MHD_CONTENT_READER_END_WITH_ERROR;
         }
 
@@ -616,11 +613,10 @@ static int request_handler_fields(
                 void *connection_cls) {
 
         _cleanup_(MHD_destroy_responsep) struct MHD_Response *response = NULL;
-        RequestMeta *m = connection_cls;
+        RequestMeta *m = ASSERT_PTR(connection_cls);
         int r;
 
         assert(connection);
-        assert(m);
 
         r = open_journal(m);
         if (r < 0)
@@ -734,7 +730,7 @@ static int request_handler_machine(
                 void *connection_cls) {
 
         _cleanup_(MHD_destroy_responsep) struct MHD_Response *response = NULL;
-        RequestMeta *m = connection_cls;
+        RequestMeta *m = ASSERT_PTR(connection_cls);
         int r;
         _cleanup_free_ char* hostname = NULL, *os_name = NULL;
         uint64_t cutoff_from = 0, cutoff_to = 0, usage = 0;
@@ -742,7 +738,6 @@ static int request_handler_machine(
         _cleanup_free_ char *v = NULL, *json = NULL;
 
         assert(connection);
-        assert(m);
 
         r = open_journal(m);
         if (r < 0)
@@ -911,7 +906,7 @@ static int parse_argv(int argc, char *argv[]) {
 
         while ((c = getopt_long(argc, argv, "hD:", options, NULL)) >= 0)
 
-                switch(c) {
+                switch (c) {
 
                 case 'h':
                         return help();
@@ -992,7 +987,7 @@ static int parse_argv(int argc, char *argv[]) {
                         return -EINVAL;
 
                 default:
-                        assert_not_reached("Unhandled option");
+                        assert_not_reached();
                 }
 
         if (optind < argc)
