@@ -12,7 +12,6 @@
 #include "sd-netlink.h"
 
 #include "in-addr-util.h"
-#include "netlink-util.h"
 #include "tests.h"
 #include "util.h"
 
@@ -30,7 +29,7 @@ static void acd_handler(sd_ipv4acd *acd, int event, void *userdata) {
                 log_error("the client was stopped");
                 break;
         default:
-                assert_not_reached("invalid ACD event");
+                assert_not_reached();
         }
 }
 
