@@ -296,10 +296,9 @@ static int wireguard_set_interface(NetDev *netdev) {
 }
 
 static int on_resolve_retry(sd_event_source *s, usec_t usec, void *userdata) {
-        WireguardPeer *peer = userdata;
+        WireguardPeer *peer = ASSERT_PTR(userdata);
         NetDev *netdev;
 
-        assert(peer);
         assert(peer->wireguard);
 
         netdev = NETDEV(peer->wireguard);
@@ -332,11 +331,10 @@ static int wireguard_peer_resolve_handler(
               const struct addrinfo *ai,
               void *userdata) {
 
-        WireguardPeer *peer = userdata;
+        WireguardPeer *peer = ASSERT_PTR(userdata);
         NetDev *netdev;
         int r;
 
-        assert(peer);
         assert(peer->wireguard);
 
         netdev = NETDEV(peer->wireguard);
@@ -405,7 +403,7 @@ static int peer_resolve_endpoint(WireguardPeer *peer) {
                 /* Not necessary to resolve the endpoint. */
                 return 0;
 
-        if (event_source_is_enabled(peer->resolve_retry_event_source) > 0)
+        if (sd_event_source_get_enabled(peer->resolve_retry_event_source, NULL) > 0)
                 /* Timer event source is enabled. The endpoint will be resolved later. */
                 return 0;
 
@@ -462,11 +460,10 @@ int config_parse_wireguard_listen_port(
                 void *data,
                 void *userdata) {
 
-        uint16_t *s = data;
+        uint16_t *s = ASSERT_PTR(data);
         int r;
 
         assert(rvalue);
-        assert(data);
 
         if (isempty(rvalue) || streq(rvalue, "auto")) {
                 *s = 0;
@@ -712,13 +709,11 @@ int config_parse_wireguard_allowed_ips(
 
                 masked = addr;
                 assert_se(in_addr_mask(family, &masked, prefixlen) >= 0);
-                if (!in_addr_equal(family, &masked, &addr)) {
-                        _cleanup_free_ char *buf = NULL;
-
-                        (void) in_addr_prefix_to_string(family, &masked, prefixlen, &buf);
+                if (!in_addr_equal(family, &masked, &addr))
                         log_syntax(unit, LOG_WARNING, filename, line, 0,
-                                   "Specified address '%s' is not properly masked, assuming '%s'.", word, strna(buf));
-                }
+                                   "Specified address '%s' is not properly masked, assuming '%s'.",
+                                   word,
+                                   IN_ADDR_PREFIX_TO_STRING(family, &masked, prefixlen));
 
                 ipmask = new(WireguardIPmask, 1);
                 if (!ipmask)
@@ -889,15 +884,13 @@ int config_parse_wireguard_route_table(
                 void *data,
                 void *userdata) {
 
-        NetDev *netdev = userdata;
-        uint32_t *table = data;
+        NetDev *netdev = ASSERT_PTR(userdata);
+        uint32_t *table = ASSERT_PTR(data);
         int r;
 
         assert(filename);
         assert(lvalue);
         assert(rvalue);
-        assert(data);
-        assert(userdata);
 
         if (isempty(rvalue) || parse_boolean(rvalue) == 0) {
                 *table = 0; /* Disabled. */
@@ -928,14 +921,13 @@ int config_parse_wireguard_peer_route_table(
                 void *userdata) {
 
         _cleanup_(wireguard_peer_free_or_set_invalidp) WireguardPeer *peer = NULL;
-        NetDev *netdev = userdata;
+        NetDev *netdev = ASSERT_PTR(userdata);
         Wireguard *w;
         int r;
 
         assert(filename);
         assert(lvalue);
         assert(rvalue);
-        assert(netdev);
         assert(netdev->manager);
 
         w = WIREGUARD(netdev);
@@ -983,13 +975,12 @@ int config_parse_wireguard_route_priority(
                 void *data,
                 void *userdata) {
 
-        uint32_t *priority = data;
+        uint32_t *priority = ASSERT_PTR(data);
         int r;
 
         assert(filename);
         assert(lvalue);
         assert(rvalue);
-        assert(data);
 
         if (isempty(rvalue)) {
                 *priority = 0;
