@@ -88,7 +88,7 @@ static int manager_link_is_online(Manager *m, Link *l, LinkOperationalStateRange
         needs_ipv4 = required_family & ADDRESS_FAMILY_IPV4;
         needs_ipv6 = required_family & ADDRESS_FAMILY_IPV6;
 
-        if (s.min >= LINK_OPERSTATE_DEGRADED) {
+        if (s.min < LINK_OPERSTATE_ROUTABLE) {
                 if (needs_ipv4 && l->ipv4_address_state < LINK_ADDRESS_STATE_DEGRADED) {
                         log_link_debug(l, "No routable or link-local IPv4 address is configured.");
                         return 0;
@@ -98,9 +98,7 @@ static int manager_link_is_online(Manager *m, Link *l, LinkOperationalStateRange
                         log_link_debug(l, "No routable or link-local IPv6 address is configured.");
                         return 0;
                 }
-        }
-
-        if (s.min >= LINK_OPERSTATE_ROUTABLE) {
+        } else {
                 if (needs_ipv4 && l->ipv4_address_state < LINK_ADDRESS_STATE_ROUTABLE) {
                         log_link_debug(l, "No routable IPv4 address is configured.");
                         return 0;
@@ -178,14 +176,13 @@ bool manager_configured(Manager *m) {
 }
 
 static int manager_process_link(sd_netlink *rtnl, sd_netlink_message *mm, void *userdata) {
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
         uint16_t type;
         Link *l;
         const char *ifname;
         int ifindex, r;
 
         assert(rtnl);
-        assert(m);
         assert(mm);
 
         r = sd_netlink_message_get_type(mm, &type);
@@ -286,7 +283,7 @@ static int manager_rtnl_listen(Manager *m) {
         if (r < 0)
                 return r;
 
-        r = sd_netlink_message_request_dump(req, true);
+        r = sd_netlink_message_set_request_dump(req, true);
         if (r < 0)
                 return r;
 
@@ -304,12 +301,10 @@ static int manager_rtnl_listen(Manager *m) {
 }
 
 static int on_network_event(sd_event_source *s, int fd, uint32_t revents, void *userdata) {
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
         Link *l;
         int r;
 
-        assert(m);
-
         sd_network_monitor_flush(m->network_monitor);
 
         HASHMAP_FOREACH(l, m->links_by_index) {
@@ -382,7 +377,7 @@ int manager_new(Manager **ret,
         (void) sd_event_add_signal(m->event, NULL, SIGINT, NULL, NULL);
 
         if (timeout > 0) {
-                r = sd_event_add_time_relative(m->event, NULL, clock_boottime_or_monotonic(), timeout, 0, NULL, INT_TO_PTR(-ETIMEDOUT));
+                r = sd_event_add_time_relative(m->event, NULL, CLOCK_BOOTTIME, timeout, 0, NULL, INT_TO_PTR(-ETIMEDOUT));
                 if (r < 0 && r != -EOVERFLOW)
                         return r;
         }
