@@ -5,27 +5,29 @@
 #include "alloc-util.h"
 #include "hexdecoct.h"
 #include "macro.h"
+#include "random-util.h"
 #include "string-util.h"
+#include "tests.h"
 
-static void test_hexchar(void) {
+TEST(hexchar) {
         assert_se(hexchar(0xa) == 'a');
         assert_se(hexchar(0x0) == '0');
 }
 
-static void test_unhexchar(void) {
+TEST(unhexchar) {
         assert_se(unhexchar('a') == 0xA);
         assert_se(unhexchar('A') == 0xA);
         assert_se(unhexchar('0') == 0x0);
 }
 
-static void test_base32hexchar(void) {
+TEST(base32hexchar) {
         assert_se(base32hexchar(0) == '0');
         assert_se(base32hexchar(9) == '9');
         assert_se(base32hexchar(10) == 'A');
         assert_se(base32hexchar(31) == 'V');
 }
 
-static void test_unbase32hexchar(void) {
+TEST(unbase32hexchar) {
         assert_se(unbase32hexchar('0') == 0);
         assert_se(unbase32hexchar('9') == 9);
         assert_se(unbase32hexchar('A') == 10);
@@ -33,13 +35,13 @@ static void test_unbase32hexchar(void) {
         assert_se(unbase32hexchar('=') == -EINVAL);
 }
 
-static void test_base64char(void) {
+TEST(base64char) {
         assert_se(base64char(0) == 'A');
         assert_se(base64char(26) == 'a');
         assert_se(base64char(63) == '/');
 }
 
-static void test_unbase64char(void) {
+TEST(unbase64char) {
         assert_se(unbase64char('A') == 0);
         assert_se(unbase64char('Z') == 25);
         assert_se(unbase64char('a') == 26);
@@ -51,26 +53,45 @@ static void test_unbase64char(void) {
         assert_se(unbase64char('=') == -EINVAL);
 }
 
-static void test_octchar(void) {
+TEST(octchar) {
         assert_se(octchar(00) == '0');
         assert_se(octchar(07) == '7');
 }
 
-static void test_unoctchar(void) {
+TEST(unoctchar) {
         assert_se(unoctchar('0') == 00);
         assert_se(unoctchar('7') == 07);
 }
 
-static void test_decchar(void) {
+TEST(decchar) {
         assert_se(decchar(0) == '0');
         assert_se(decchar(9) == '9');
 }
 
-static void test_undecchar(void) {
+TEST(undecchar) {
         assert_se(undecchar('0') == 0);
         assert_se(undecchar('9') == 9);
 }
 
+static void test_hexmem_one(const char *in, const char *expected) {
+        _cleanup_free_ char *result = NULL;
+        _cleanup_free_ void *mem = NULL;
+        size_t len;
+
+        assert_se(result = hexmem(in, strlen_ptr(in)));
+        log_debug("hexmem(\"%s\") → \"%s\" (expected: \"%s\")", strnull(in), result, expected);
+        assert_se(streq(result, expected));
+
+        assert_se(unhexmem(result, SIZE_MAX, &mem, &len) >= 0);
+        assert_se(memcmp_safe(mem, in, len) == 0);
+}
+
+TEST(hexmem) {
+        test_hexmem_one(NULL, "");
+        test_hexmem_one("", "");
+        test_hexmem_one("foo", "666f6f");
+}
+
 static void test_unhexmem_one(const char *s, size_t l, int retval) {
         _cleanup_free_ char *hex = NULL;
         _cleanup_free_ void *mem = NULL;
@@ -84,12 +105,12 @@ static void test_unhexmem_one(const char *s, size_t l, int retval) {
                         l = strlen(s);
 
                 assert_se(hex = hexmem(mem, len));
-                answer = strndupa(strempty(s), l);
+                answer = strndupa_safe(strempty(s), l);
                 assert_se(streq(delete_chars(answer, WHITESPACE), hex));
         }
 }
 
-static void test_unhexmem(void) {
+TEST(unhexmem) {
         const char *hex = "efa2149213";
         const char *hex_space = "  e f   a\n 2\r  14\n\r\t9\t2 \n1\r3 \r\r\t";
         const char *hex_invalid = "efa214921o";
@@ -108,7 +129,7 @@ static void test_unhexmem(void) {
 }
 
 /* https://tools.ietf.org/html/rfc4648#section-10 */
-static void test_base32hexmem(void) {
+TEST(base32hexmem) {
         char *b32;
 
         b32 = base32hexmem("", STRLEN(""), true);
@@ -190,12 +211,12 @@ static void test_unbase32hexmem_one(const char *hex, bool padding, int retval, c
         if (retval == 0) {
                 char *str;
 
-                str = strndupa(mem, len);
+                str = strndupa_safe(mem, len);
                 assert_se(streq(str, ans));
         }
 }
 
-static void test_unbase32hexmem(void) {
+TEST(unbase32hexmem) {
         test_unbase32hexmem_one("", true, 0, "");
 
         test_unbase32hexmem_one("CO======", true, 0, "f");
@@ -243,7 +264,7 @@ static void test_unbase32hexmem(void) {
 }
 
 /* https://tools.ietf.org/html/rfc4648#section-10 */
-static void test_base64mem(void) {
+TEST(base64mem) {
         char *b64;
 
         assert_se(base64mem("", STRLEN(""), &b64) == 0);
@@ -275,6 +296,151 @@ static void test_base64mem(void) {
         free(b64);
 }
 
+TEST(base64mem_linebreak) {
+        uint8_t data[4096];
+
+        for (size_t i = 0; i < 20; i++) {
+                _cleanup_free_ char *encoded = NULL;
+                _cleanup_free_ void *decoded = NULL;
+                size_t decoded_size;
+                uint64_t n, m;
+                ssize_t l;
+
+                /* Try a bunch of differently sized blobs */
+                n = random_u64_range(sizeof(data));
+                random_bytes(data, n);
+
+                /* Break at various different columns */
+                m = 1 + random_u64_range(n + 5);
+
+                l = base64mem_full(data, n, m, &encoded);
+                assert_se(l >= 0);
+                assert_se(encoded);
+                assert_se((size_t) l == strlen(encoded));
+
+                assert_se(unbase64mem(encoded, SIZE_MAX, &decoded, &decoded_size) >= 0);
+                assert_se(decoded_size == n);
+                assert_se(memcmp(data, decoded, n) == 0);
+
+                for (size_t j = 0; j < (size_t) l; j++)
+                        assert_se((encoded[j] == '\n') == (j % (m + 1) == m));
+        }
+}
+
+static void test_base64_append_one(char **buf, size_t *len, const char *in, const char *expected) {
+        ssize_t new_len;
+
+        new_len = base64_append(buf, *len, in, strlen_ptr(in), 8, 12);
+        assert_se(new_len >= 0);
+        log_debug("base64_append_one(\"%s\")\nresult:\n%s\nexpected:\n%s", in, strnull(*buf), strnull(expected));
+        assert_se((size_t) new_len == strlen_ptr(*buf));
+        assert_se(streq_ptr(*buf, expected));
+        *len = new_len;
+}
+
+TEST(base64_append) {
+        _cleanup_free_ char *buf = NULL;
+        size_t len = 0;
+
+        test_base64_append_one(&buf, &len, "", NULL);
+        test_base64_append_one(&buf, &len, "f",
+                               "Zg==");
+        test_base64_append_one(&buf, &len, "fo",
+                               "Zg== Zm8=");
+        test_base64_append_one(&buf, &len, "foo",
+                               "Zg== Zm8=\n"
+                               "        Zm9v");
+        test_base64_append_one(&buf, &len, "foob",
+                               "Zg== Zm8=\n"
+                               "        Zm9v\n"
+                               "        Zm9v\n"
+                               "        Yg==");
+        test_base64_append_one(&buf, &len, "fooba",
+                               "Zg== Zm8=\n"
+                               "        Zm9v\n"
+                               "        Zm9v\n"
+                               "        Yg==\n"
+                               "        Zm9v\n"
+                               "        YmE=");
+        test_base64_append_one(&buf, &len, "foobar",
+                               "Zg== Zm8=\n"
+                               "        Zm9v\n"
+                               "        Zm9v\n"
+                               "        Yg==\n"
+                               "        Zm9v\n"
+                               "        YmE=\n"
+                               "        Zm9v\n"
+                               "        YmFy");
+
+        assert_se(free_and_strdup(&buf, "hogehogehogehoge") >= 0);
+        len = strlen(buf);
+
+        test_base64_append_one(&buf, &len, "",
+                               "hogehogehogehoge");
+        test_base64_append_one(&buf, &len, "f",
+                               "hogehogehogehoge\n"
+                               "        Zg==");
+        test_base64_append_one(&buf, &len, "fo",
+                               "hogehogehogehoge\n"
+                               "        Zg==\n"
+                               "        Zm8=");
+        test_base64_append_one(&buf, &len, "foo",
+                               "hogehogehogehoge\n"
+                               "        Zg==\n"
+                               "        Zm8=\n"
+                               "        Zm9v");
+        test_base64_append_one(&buf, &len, "foob",
+                               "hogehogehogehoge\n"
+                               "        Zg==\n"
+                               "        Zm8=\n"
+                               "        Zm9v\n"
+                               "        Zm9v\n"
+                               "        Yg==");
+        test_base64_append_one(&buf, &len, "fooba",
+                               "hogehogehogehoge\n"
+                               "        Zg==\n"
+                               "        Zm8=\n"
+                               "        Zm9v\n"
+                               "        Zm9v\n"
+                               "        Yg==\n"
+                               "        Zm9v\n"
+                               "        YmE=");
+        test_base64_append_one(&buf, &len, "foobar",
+                               "hogehogehogehoge\n"
+                               "        Zg==\n"
+                               "        Zm8=\n"
+                               "        Zm9v\n"
+                               "        Zm9v\n"
+                               "        Yg==\n"
+                               "        Zm9v\n"
+                               "        YmE=\n"
+                               "        Zm9v\n"
+                               "        YmFy");
+
+        assert_se(free_and_strdup(&buf, "hogehogehogehoge") >= 0);
+        len = strlen(buf);
+
+        test_base64_append_one(&buf, &len, "foobarfoobarfoobarfoobar",
+                               "hogehogehogehoge\n"
+                               "        Zm9v\n"
+                               "        YmFy\n"
+                               "        Zm9v\n"
+                               "        YmFy\n"
+                               "        Zm9v\n"
+                               "        YmFy\n"
+                               "        Zm9v\n"
+                               "        YmFy");
+
+        assert_se(free_and_strdup(&buf, "aaa") >= 0);
+        len = strlen(buf);
+
+        test_base64_append_one(&buf, &len, "foobarfoobarfoobarfoobar",
+                               "aaa Zm9vYmFy\n"
+                               "    Zm9vYmFy\n"
+                               "    Zm9vYmFy\n"
+                               "    Zm9vYmFy");
+}
+
 static void test_unbase64mem_one(const char *input, const char *output, int ret) {
         _cleanup_free_ void *buffer = NULL;
         size_t size = 0;
@@ -288,7 +454,7 @@ static void test_unbase64mem_one(const char *input, const char *output, int ret)
         }
 }
 
-static void test_unbase64mem(void) {
+TEST(unbase64mem) {
 
         test_unbase64mem_one("", "", 0);
         test_unbase64mem_one("Zg==", "f", 0);
@@ -314,7 +480,7 @@ static void test_unbase64mem(void) {
         test_unbase64mem_one(" Z m 8 = q u u x ", NULL, -ENAMETOOLONG);
 }
 
-static void test_hexdump(void) {
+TEST(hexdump) {
         uint8_t data[146];
         unsigned i;
 
@@ -333,23 +499,4 @@ static void test_hexdump(void) {
         hexdump(stdout, data, sizeof(data));
 }
 
-int main(int argc, char *argv[]) {
-        test_hexchar();
-        test_unhexchar();
-        test_base32hexchar();
-        test_unbase32hexchar();
-        test_base64char();
-        test_unbase64char();
-        test_octchar();
-        test_unoctchar();
-        test_decchar();
-        test_undecchar();
-        test_unhexmem();
-        test_base32hexmem();
-        test_unbase32hexmem();
-        test_base64mem();
-        test_unbase64mem();
-        test_hexdump();
-
-        return 0;
-}
+DEFINE_TEST_MAIN(LOG_INFO);
