@@ -11,10 +11,10 @@
 #include "strv.h"
 #include "tests.h"
 
-static void test_terminal_urlify(void) {
+TEST(terminal_urlify) {
         _cleanup_free_ char *formatted = NULL;
 
-        assert_se(terminal_urlify("https://www.freedesktop.org/wiki/Software/systemd/", "systemd homepage", &formatted) >= 0);
+        assert_se(terminal_urlify("https://www.freedesktop.org/wiki/Software/systemd", "systemd homepage", &formatted) >= 0);
         printf("Hey, consider visiting the %s right now! It is very good!\n", formatted);
 
         formatted = mfree(formatted);
@@ -23,7 +23,7 @@ static void test_terminal_urlify(void) {
         printf("Or click on %s to have a look at it!\n", formatted);
 }
 
-static void test_cat_files(void) {
+TEST(cat_files) {
         assert_se(cat_files("/no/such/file", NULL, 0) == -ENOENT);
         assert_se(cat_files("/no/such/file", NULL, CAT_FLAGS_MAIN_FILE_OPTIONAL) == 0);
 
@@ -31,13 +31,21 @@ static void test_cat_files(void) {
                 assert_se(cat_files("/etc/fstab", STRV_MAKE("/etc/fstab", "/etc/fstab"), 0) == 0);
 }
 
-int main(int argc, char *argv[]) {
-        test_setup_logging(LOG_INFO);
+TEST(red_green_cross_check_mark) {
+        bool b = false;
 
-        test_terminal_urlify();
-        test_cat_files();
+        printf("yea: <%s>\n", GREEN_CHECK_MARK());
+        printf("nay: <%s>\n", RED_CROSS_MARK());
 
-        print_separator();
-
-        return 0;
+        printf("%s → %s → %s → %s\n",
+               COLOR_MARK_BOOL(b),
+               COLOR_MARK_BOOL(!b),
+               COLOR_MARK_BOOL(!!b),
+               COLOR_MARK_BOOL(!!!b));
 }
+
+TEST(print_separator) {
+        print_separator();
+}
+
+DEFINE_TEST_MAIN(LOG_INFO);
