@@ -99,8 +99,7 @@ static bool wall_tty_match(const char *path, bool is_local, void *userdata) {
         _cleanup_close_ int fd = -1;
         struct stat st;
 
-        if (!path_is_absolute(path))
-                path = strjoina("/dev/", path);
+        assert(path_is_absolute(path));
 
         if (lstat(path, &st) < 0) {
                 log_debug_errno(errno, "Failed to stat %s: %m", path);
